import { AsyncCallback, AsyncCb, BaseCallback, Callback, Cb, GraphMode, NodeFilterFn, NodeType, PartialConfig, PositionType, TraversalMode } from "./types";
import { WalkNode } from "./node";
declare class CallbacksBuilder<T extends BaseCallback, CbType extends Cb, TUpper extends BaseWalkBuilder<T, CbType>> {
    private cbs;
    private source;
    private readonly callback;
    constructor(cbs: CbType[], source: TUpper);
    withExecutionOrder(order: number): CallbacksBuilder<T, CbType, TUpper>;
    withFilter(fn: NodeFilterFn): this;
    withFilters(...fn: NodeFilterFn[]): this;
    filteredByNodeTypes(...types: NodeType[]): CallbacksBuilder<T, CbType, TUpper>;
    filteredByKeys(...keys: string[]): CallbacksBuilder<T, CbType, TUpper>;
    filteredByPosition(position: PositionType): CallbacksBuilder<T, CbType, TUpper>;
    done(): TUpper;
}
declare abstract class BaseWalkBuilder<T extends BaseCallback, CbType extends Cb> {
    protected _config: PartialConfig<T>;
    private globalFilters;
    resetConfig(): this;
    withConfig(config: PartialConfig<T>): this;
    withTraversalMode(traversalMode: TraversalMode): this;
    withGraphMode(graphMode: GraphMode): this;
    withRootObjectCallbacks(val: boolean): this;
    withRunningCallbacks(val: boolean): this;
    withConfiguredCallbacks(...callbacks: CbType[]): CallbacksBuilder<T, CbType, this>;
    withConfiguredCallback(callback: CbType): CallbacksBuilder<T, CbType, this>;
    withCallback(callback: T): this;
    withGlobalFilter(fn: NodeFilterFn): this;
    withCallbacks(...callbacks: T[]): this;
    getCurrentConfig(): PartialConfig<T>;
}
export declare class WalkBuilder extends BaseWalkBuilder<Callback, Cb> {
    walk(obj: object): void;
    walkStep(obj: object): Generator<WalkNode>;
    withSimpleCallback(callback: Cb): this;
    withSimpleCallbacks(...callbacks: Cb[]): this;
}
export declare class AsyncWalkBuilder extends BaseWalkBuilder<AsyncCallback, AsyncCb> {
    walk(obj: object): Promise<void>;
    walkStep(obj: object): AsyncGenerator<WalkNode>;
    withParallelizeAsyncCallbacks(val: boolean): this;
    withSimpleCallback(callback: AsyncCb): this;
    withSimpleCallbacks(...callbacks: AsyncCb[]): this;
}
export {};
